<?php
/**
 * Zarafa Z-Merge SugarCRM Connector
 *
 * Copyright (C) 2005 - 2008  Zarafa B.V.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License, version 3,
 * and under the terms of the GNU General Public License, version 3,
 * as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU (Affero) General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * and the GNU General Public License along with this program.
 * If not, see <http://www.gnu.org/licenses/>.
 *
 * Created on 24.10.2008 by Sebastian Kummer & Manfred Kutas
 *
 * @package Z-Merge
 * @subpackage SugarCRM-Connector
 */

class ZMergeController extends SugarController {

    /**
    * Constructor
    */
    function ZMergeController(){
        parent::SugarController();
    }


    /**
    * Invoked from the parent to process the requested action
    *
    * @return void
    */
    public function process(){
        $this->do_action = $this->action;
        $this->handle_action();
        // do nothing after this point
        $this->view = "";
    }


    /**
    * Action to perform the serverside 'check'
    *
    * @related-to: Z-Merge-Agent
    * @return void
    */
    public function action_check() {
        include("modules/ZMerge/serverside.check.php");
    }


    /**
    * Action to exetute the serverside connector performing read/write operations
    *
    * @related-to: Z-Merge-Agent
    * @return void
    */
    public function action_connector() {
        include("modules/ZMerge/serverside.connector.php");
    }


    /**
    * Action to perform the serverside 'archiveMail'
    * (only saves raw RFC822 message as file)
    *
    * @related-to: Webaccess-ZMA plugin (right mouseclick archive-to)
    * @return void
    */
    public function action_archivemail() {
        include("modules/ZMerge/serverside.archiveMail.php");
    }


    /**
    * Popup/Window to be seen so that a user can assign related objects to the (not yet) archived-Mail
    *
    * @related-to: Webaccess-ZMA plugin (right mouseclick archive-to)
    * @workflow: entry point, step 1
    * @return void
    */
    public function action_archivepopup() {
        $view = new SugarView();
        // SugarCRM 6 fix: 5 versions need this call, 6 don't have it
        if(method_exists($view, "displayJavascript"))
            $view->displayJavascript();
        include("modules/ZMerge/gui.archivePopup.php");
    }


    /**
    * Action reads a temporary RFC822 message file,
    * parses it contents and creates an SugarCRM Email object and
    * creates and relates SugarCRM Notes objects for the attachments.
    *
    * @related-to: Webaccess-ZMA plugin (right mouseclick archive-to)
    * @workflow: after selection related owner, assigned-to etc, step 2
    * @return void
    */
    public function action_archiveaction() {
        $view = new SugarView();
        // SugarCRM 6 fix: 5 versions need this call, 6 don't have it
        if(method_exists($view, "displayJavascript"))
            $view->displayJavascript();

        include("modules/ZMerge/gui.archiveAction.php");
    }


    /**
    * Action shows status of the archived message and offers a button
    * to close the popup
    *
    * @related-to: Webaccess-ZMA plugin (right mouseclick archive-to)
    * @workflow: status window, step 3
    * @return void
    */
    public function action_archiveclose() {
        $view = new SugarView();
        // SugarCRM 6 fix: 5 versions need this call, 6 don't have it
        if(method_exists($view, "displayJavascript"))
            $view->displayJavascript();

        include("modules/ZMerge/gui.archiveClose.php");
    }


    /**
    * Default action when loggin in
    *
    * @related-to: Z-Merge-Agent, Webaccess-ZMA plugin (right mouseclick archive-to)
    * @workflow: authentication-layer
    * @return void
    */
    public function action_justBe() {
        echo "I am!";
    }

}
?>