<?php
/**
 * Zarafa Z-Merge SugarCRM Connector
 *
 * Copyright (C) 2005 - 2008  Zarafa B.V.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License, version 3,
 * and under the terms of the GNU General Public License, version 3,
 * as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU (Affero) General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * and the GNU General Public License along with this program.
 * If not, see <http://www.gnu.org/licenses/>.
 *
 * Created on 10.06.2008 by Sebastian Kummer & Manfred Kutas
 *
 * NOTE: This action reads a temporary RFC822 message file,
 * parses it contents and creates an SugarCRM Email object and
 * creates and relates SugarCRM Notes objects for the attachments.
 *
 * @requires Pear-Mail
 * @package Z-Merge
 * @subpackage SugarCRM-Connector
 */



if(!defined('sugarEntry') || !sugarEntry) die('Not A Valid Entry Point');

require_once('data/SugarBean.php');
require_once('modules/Emails/Email.php');
require_once('modules/Notes/Note.php');
require_once('include/utils.php');

require_once('modules/ZMerge/include/mimeDecode.php');
include_once('include/z_RFC822.php');
require_once('inc.utils.php');

global $timedate, $sugar_config, $current_user;


// try to open the file with the email
$tmpAttachPath = $sugar_config['tmp_dir'];
$mailid = stripslashes($_REQUEST['record']);

$filename = $sugar_config['upload_dir'] . stripslashes($_REQUEST['record']) . ".eml";
if (file_exists($filename)) {
    $bean = new Email();

    if(empty($bean->id)){
        $bean->id = create_guid();
        $bean->new_with_id = true;
    }

    // read and parse the email
    $rfc822 = file_get_contents($filename);
    $mobj = new Mail_mimeDecode($rfc822);
    $message = $mobj->decode(array('decode_headers' => true, 'decode_bodies' => true, 'include_bodies' => true, 'input' => $rfc822, 'crlf' => "\n", 'charset' => 'utf-8'));

    // create bean
    $bean->assigned_user_id = $_REQUEST['assigned_user_id'];
    $bean->assigned_user_name = $_REQUEST['assigned_user_name'];

    $bean->parent_type = $_REQUEST['parent_type'];
    $bean->parent_id = $_REQUEST['parent_id'];

    // create a relation between the objects
    $bean->load_relationships();
    $parenttype = strtolower($_REQUEST['parent_type']);
    if (isset($bean->$parenttype)) {
        $bean->$parenttype->add($_REQUEST['parent_id']);
    }

    // assign email variables
    $receivetime = gmdate($GLOBALS['timedate']->get_db_date_time_format(), strtotime($message->headers["date"]));
    $bean->date_start = $timedate->to_display_date($receivetime);
    $bean->time_start = $timedate->to_display_time($receivetime, true);
    $bean->date_sent = "";

    $bean->name = ($message->headers["subject"])?$message->headers["subject"]:'n/a';

    if(isset($message->headers["to"])) {
        $bean->to_addrs_emails = parseAddr(Mail_RFC822::parseAddressList($message->headers["to"]));
        $bean->to_addrs = parseAddrName(Mail_RFC822::parseAddressList($message->headers["to"]));;
    }

    if(isset($message->headers["cc"])) {
        $bean->cc_addrs_emails = parseAddr(Mail_RFC822::parseAddressList($message->headers["cc"]));
        $bean->cc_addrs = parseAddrName(Mail_RFC822::parseAddressList($message->headers["cc"]));;
    }

    if(isset($message->headers["bcc"])) {
        $bean->bcc_addrs_emails = parseAddr(Mail_RFC822::parseAddressList($message->headers["bcc"]));
        $bean->bcc_addrs = parseAddrName(Mail_RFC822::parseAddressList($message->headers["bcc"]));;
    }

    if(isset($message->headers["from"])) {
        $bean->from_addr_name = parseAddr(Mail_RFC822::parseAddressList($message->headers["from"]));
        $bean->from_addr = parseAddrName(Mail_RFC822::parseAddressList($message->headers["from"]));
        $bean->from_name = $bean->from_addr;
    }

    $bean->description = "";
    $bean->description_html = "";
    getBodyRecursive($message, "plain", &$bean->description);
    getBodyRecursive($message, "html", &$bean->description_html);

    // Attachments are only searched in the top-level part
    $attachments = "";
    $n = 0;
    if(isset($message->parts)) {
        foreach($message->parts as $part) {
            $att = array();
            if(in_array($n, $_REQUEST['attachment']) && isset($part->disposition) && ($part->disposition == "attachment" || $part->disposition == "inline")) {

                $tmpfile = $tmpAttachPath. "/". $mailid . $n;
                // write the body to a file
                if (isset($part->body)) {
                    @$fp = fopen($tmpfile,"wb");
                    @fwrite($fp, $part->body);
                    @fclose($fp);
                }

                if(isset($part->d_parameters['filename']))
                    $attname = $part->d_parameters['filename'];
                else if(isset($part->ctype_parameters['name']))
                    $attname = $part->ctype_parameters['name'];
                else if(isset($part->headers['content-description']))
                    $attname = $part->headers['content-description'];
                else $attname = "unknown attachment";

                // create attachment and move file to its final location
                $GLOBALS['log']->fatal( "create attachment $attname $tmpfile  ". $bean->id );
                $att = createObjectsForAttachment($bean, $attname,$tmpfile);
            }
            $n++;
        }
    }

    $bean->type = "archived";
    $bean->status = "archived";
    $bean->save();

    $record_id = $bean->id;
       $GLOBALS['log']->info("ZMerge:gui.archiveAction EmailBean saved - exiting $record_id");

       //delete the temp file
       unlink($filename);
}
else {
    echo "error: temporarily email file not found";
    exit();
}


// forward to the next action/screen
if (isset($_POST['edit_before_saving']) && $_POST['edit_before_saving'] == 1)
    $donext = "EditView&module=Emails";
else
    $donext = "archiveClose&module=ZMerge";

if ($record_id)
    header("Location: index.php?action=$donext&record=$record_id");
else
    echo "error - nothing archived";






function createObjectsForAttachment(&$bean, $cleanAttachmentFileName, $tmpfilename) {
    global $tmpAttachPath, $sugar_config;

    $cleanAttachmentFileName = str_replace("\\", "", $cleanAttachmentFileName);
    $GLOBALS['log']->fatal("ZMerge::createObjectsForAttachment: Email Attachment [ {$cleanAttachmentFileName} ] ");

    $badExt = $bean->safeAttachmentName($cleanAttachmentFileName);
    if ($badExt) {
        $destination = $destination . ".txt";
    } // if
    $fileName = $badExt ? $cleanAttachmentFileName . ".txt" : $cleanAttachmentFileName;

    // create note
    $note = new Note();
    $note->id = create_guid();
    $note->new_with_id = true; // duplicating the note with files
    $note->parent_id = $bean->id;
    $note->parent_type = $bean->module_dir;
    $note->name = "Email Attachment: " .$cleanAttachmentFileName;
    $note->filename = $cleanAttachmentFileName;
    $noteFile = "{$sugar_config['upload_dir']}{$note->id}";

    // move file to its final destination
    $GLOBALS['log']->fatal("ZMerge::createObjectsForAttachment: from:  $tmpfilename ($cleanAttachmentFileName)");
    $GLOBALS['log']->fatal("ZMerge::createObjectsForAttachment: notefile:". $noteFile);
    if(!rename($tmpfilename, $noteFile)) {
        $GLOBALS['log']->fatal("ZMerge::createObjectsForAttachment: Email Attachment [ {$tmpfilename} ] could not be moved to upload dir");
        return false;
    }

    // get mime type and save
    $note->file_mime_type = $bean->email2GetMime($noteFile);
    $note->save();

    // add Note to attachments array
    $bean->attachments[] = $note;
    return true;
}


?>