<?php
/**
 * Zarafa Z-Merge SugarCRM Connector
 *
 * Copyright (C) 2005 - 2008  Zarafa B.V.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License, version 3,
 * and under the terms of the GNU General Public License, version 3,
 * as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU (Affero) General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * and the GNU General Public License along with this program.
 * If not, see <http://www.gnu.org/licenses/>.
 *
 * Created on 22.04.2008 by Sebastian Kummer & Manfred Kutas
 *
 * @package Z-Merge
 * @subpackage SugarCRM-Connector
 */


/**
* getChanges()
* Returns changes made in this system since $modid
*
* @param String     $session            legacy - ignored
* @param String     $modid              Modification-ID
* @param boolean    $completeChanges    if true, all changes are immediatly returned incl. changes made by Z-Merge. Don't wait for events
* @return complex-array()
*/
function getChanges ($session, $modid, $completeChanges = false) {
    global $db, $sugar_config, $current_user, $wait_for_event;

    $modid = intval($modid);

    $time = 0;
    $changes = array();

    // if only complete changes are requested, don't wait for an event
    if ($completeChanges) $wait_for_event = 0;

    while ((count($changes) == 0) && ($time <= $wait_for_event))    {
        if (!$db) $db = & DBManager :: getInstance();

        // if complete changes are requested, send ALL changes, including changes done by Z-Merge itself (necessary for off synching)
        if ($completeChanges)     $chtype = "";
        else                    $chtype = " AND zmerge = 0";

        $stmt = "SELECT DISTINCT(crmid) as crmid, MAX(id) as id, modified, type " .
                "FROM zmerge WHERE id > $modid $chtype GROUP BY crmid ORDER BY ID";
        $result = $db->query($stmt);
        if (!empty($result)) {
            $cnt = 0;
            while ($row = $db->fetchByAssoc($result)) {

                // get data for object
                $data = getObject($row['crmid'], $row['type']);
                if ($data) {
                    $changes[$cnt]['modid'] = $row['id'];
                    $changes[$cnt]['id'] = $row['crmid'];
                    $changes[$cnt]['modtime'] = strtotime($row['modified']);
                    $changes[$cnt]['objtype'] = $row['type'];
                    $changes[$cnt]['data'] = $data;

                    // only transmit a configured number of changes at one time
                    if (($cnt+1) >= SG_MAX_OBJECTS) {
                        $changes[$cnt]['morechanges'] = true;
                        break;
                    }

                    $cnt++;
                }
            }
        }
        // no events raised, wait...
        if (count($changes) == 0) {
            usleep(250000);
            $time = $time + 0.25;
        }
    }

    return $changes;
}


/**
* setChanges
* applies changes to the server
*
* @param String $session legacy - ignored
* @param complex-array() $objects Objects to modify
* @param boolean $isConflict    if true, all objects are created. The transmitted object-id is from the original object (for better cloning meta-data)
* @return complex-array()
*/
function setChanges ($session, $objects, $isConflict = false) {
    global $db, $sugar_config, $current_user, $wait_for_event;

    if (! $objects) return;

    // if these are conflicts, they have to be (!!!) replied back as NEW on the next sync!
    if ($isConflict)
        $GLOBALS['zmerge_change'] = 0;

    $newobj = array();
    $cnt = 0;
    foreach ($objects as $obj) {
        // initialize vars
        $stat = "";
        $message = "";
        $crmid = "";

        $type = $obj["objtype"];
        // special treatment for appointments
        if ($type == "appointments") {
            // save new appointments as meetings
            if ($obj["id"]) {
                // look in the DB, if that appointment is a meeting or a call
                $sql = "SELECT 'calls' as type FROM calls WHERE calls.id = '".$obj["id"]."'  UNION
                SELECT 'meetings' as type FROM meetings WHERE meetings.id = '".$obj["id"]."'";

                $result = $db->query($sql);
                if (!empty($result)) {
                    $row = $db->fetchByAssoc($result);
                    $type = $row['type'];
                }
                else
                    $type = "meetings";
            }
            // new appointments as default "Meeting"
            else {
                $type = "meetings";
            }
        }

        // deleterequest
        if ($obj["delete"] == 1) {
            if ($obj["userid"] == "") {
                $stat = "delete_fail";
                $message = "no userid set!";
            }
            else {
                // delete object
                $status = deleteObject($obj["id"], $type);

                if (strpos($status, "error") === false) {
                    $stat = "delete_ok";
                    $crmid = $obj["id"];
                }
                else {
                    $stat = "delete_fail";
                    $crmid = $obj["id"];
                }
                $message = $status;
            }
        }
        // create or update
        else {
            if ($obj["userid"] == "") {
                if ($obj["id"] != "") $stat = "update_fail";
                else                  $stat = "create_fail";
                $message = "no userid set!";
            }
            else {
                // if conflict, create as new
                if ($isConflict) $obj["id"] = "";

                $status = setObject($obj["id"], $type , $obj["data"]);

                // error creating/modifying the object
                if (strpos($status, "error") === false) {
                    $crmid = $status;

                    if ($status == $obj["id"]) {
                        $stat = "update_ok";
                        $message = "successfully updated object: ". $crmid;
                    }
                    else {
                        $stat = "create_ok";
                        $message = "successfully created object: ". $crmid;
                    }
                }
                else {
                    if ($obj["id"] == "") $stat = "create_fail";
                    else                  $stat = "update_fail";

                    $message = $status;
                    $crmid = "";
                }
            }
        }

        $newobj[$cnt]['modid'] = $obj["modid"];
        $newobj[$cnt]['id'] = $crmid;
        $newobj[$cnt]['userid'] = $obj["userid"];
        $newobj[$cnt]['uid'] = $obj["uid"];
        $newobj[$cnt]['status'] = $stat;
        $newobj[$cnt]['message'] = $message;

        $cnt++;

    }

    // reset relog
    if ($isConflict)
        $GLOBALS['zmerge_change'] = 1;

    return $newobj;
}


/**
* Ping function.
* Returns true if web and db servers are working
*
* @return boolean
*/
function ping() {
    global $db;

    if ($db && checkDatabase())
        return array('alive' => true);
    else
        return array('alive' => false);
}


/**
* getObject() function.
* serializes a SugarCRM object
*
* @param String $crmID    CRM-id of the object
* @param String $type     Type of the object
* @return complex-array()
*/
function getObject($crmID, $type) {
    global $current_user, $timedate;

    $user = "";

    // The connector may return several distict objects for a single sugar-object (ex. related objects)
    $do = array();
    if ($type == "tasks") $seed = new Task();
    if ($type == "contacts") $seed = new Contact();
    if ($type == "notes") $seed = new Note();
    if ($type == "calls") $seed = new Call();
    if ($type == "meetings") $seed = new Meeting();

    if (isset($seed)) {
        $d = array();

        // load object
        if ($crmID)    $seed->retrieve($crmID, true, false);

        $out = array();
        $output_username = false;
        foreach( get_field_list($seed) as $key) {
            $value = $seed->$key['name'];
            if ($value || $value == "") {
                $out[] = array('name' => $key['name'], 'value' => $value);
            }
        }

        // add the username if available
        if (isset($seed->assigned_user_id))
           $out[] = array('name' => "sugar_user_name", 'value' => getUserName($seed->assigned_user_id));

        // add type specific values
        if ($type == "calls" || $type == "meetings" || $type == "tasks") {
            // calculate UTC start_time according with locale settings as timestamp
            $starttime = getUTC($timedate->to_db($seed->date_start ." ". $seed->time_start));

            $out[] = array('name' => 'utc_timestamp_start', 'value' => $starttime);

            if ($type == "tasks")
                $out[] = array('name' => 'utc_timestamp_due',   'value' => getUTC($timedate->to_db($seed->date_due ." ". $seed->time_due)));

            if ($type == "calls" || $type == "meetings") {
                $endtime = $starttime + ( (($seed->duration_hours * 60) + $seed->duration_minutes) * 60 );
                $out[] = array('name' => 'utc_timestamp_end',   'value' => $endtime);
            }
        }

        // convert the birthdate
        if (isset($seed->birthdate)) {
            $out[] = array('name' => 'utc_birthdate',   'value' => getUTC($timedate->to_db($seed->birthdate)));
        }

        // SugarCRM 5 - get email adresses
        if (isset($seed->emailAddress)) {
            $c = 1;
            foreach ($seed->emailAddress->addresses as $k=>$address) {
                $out[] = array('name' => 'email' . $c,   'value' => $address['email_address']);
                $c++;
                if ($c == 3) break;
            }
        }

        // get the deleted flag from seed
        $out[] = array('name' => "deleted", 'value' => $seed->deleted);

        // Notes don't have an "assinged_user_id". Use "created_by" instead
        if ($type == "notes")
            $out[] = array('name' => "sugar_user_name", 'value' => getUserName($seed->created_by));


        if ($type == "calls" || $type== "meetings")
            $d['objtype'] = "appointments";
        else
            $d['objtype'] = $type;

        $d['name_value_array'] = $out;

        $do[] = $d;
    }
    return $do;
}


/**
* setObject() function.
* unserializes a SugarCRM object
*
* @param String $crmID    CRM-id of the object
* @param String $type     Type of the object
* @param complex-array() $data Serialized object data
* @return complex-array()
*/
function setObject($crmID, $type, $data) {
    global $current_user, $timedate;

    // array of sugar-objects
    if     ($type == "tasks")      $seed = new Task();
    elseif ($type == "contacts") $seed = new Contact();
    elseif ($type == "notes")      $seed = new Note();
    elseif ($type == "calls")      $seed = new Call();
    elseif ($type == "meetings") $seed = new Meeting();
    else return "error: object type($type) not implemented";

    if (isset($seed)) {
        $d = array();

        // load object
        if ($crmID)
            $objf = $seed->retrieve($crmID);

        // check, if obj loaded correctly
        if (! isset($objf)) $crmID = null;

        $out = array();
        // value is needed for calculation of endtime for Meetings/Calls
        $starttime = 0;
        foreach( $data[0] as $kvpair) {
            $var   = $kvpair['name'];
            $value = $kvpair['value'];


            // handle special attributes
            // all utc_ variables (timestamps)
            if (stripos($var, "utc_") !== false) {

                if ($var == "utc_timestamp_start") {
                    if ($value == "") {
                        $seed->date_start_flag = 1;
                        $seed->date_start = "";
                        $seed->time_start = "";
                    }
                    else {
                        $seed->date_start_flag = 0;
                        $seed->date_start = $timedate->to_display_date_time(gmdate('Y-m-d H:i:s', $value));
                        $seed->time_start = $timedate->to_display_time(gmdate('Y-m-d H:i:s', $value), true);
                        $starttime = $value;
                    }
                }
                elseif($var == "utc_timestamp_end" && $starttime) {
                    // calculate the duration in hours and minutes
                    $offset = $value - $starttime;
                    $m = floor($offset/60);
                    $h = floor($m/60);
                    $min = $m - ($h * 60);

                    $seed->duration_hours = $h;
                    $seed->duration_minutes = $min;
                }
                else if ($var == "utc_timestamp_due") {
                    if ($value == "") {
                        $seed->date_due_flag = 1;
                        $seed->date_due = "";
                        $seed->time_due = "";
                    }
                    else {
                        $seed->date_due_flag = 0;
                        $seed->date_due = $timedate->to_display_date_time(gmdate('Y-m-d H:i:s', $value));
                        $seed->time_due = $timedate->to_display_time(gmdate('Y-m-d H:i:s', $value), true);
                    }
                }
                else if ($var == "utc_birthdate") {
                    if ($value == "")
                        $seed->birthdate = "";
                    else
                        $seed->birthdate = $timedate->to_display_date_time(gmdate('Y-m-d H:i:s', $value));
                }
            }
            // assign username from id
            else if ($var == "assigned_user_name") {
                $userid = getUserId($value);
                // Assign created_by when creating
                if (! $crmID) {
                    $seed->set_created_by = false;
                    $seed->created_by = $userid;
                }

                // Only "Note" doesn't support an assigned user
                if ($type != "notes") {
                    $seed->assigned_user_id = $userid;
                    $seed->update_modified_by = false;
                    $seed->modified_user_id = $userid;
                }
            }

            // set all "normal" variables
            else {
                $seed->$var = utf8_encode($value);
            }
        }
        // some modules don't return the id on overwriting the save()-method :-(
        $seed->save();
        return $seed->id;
    }
    return "error: object could not instantiated : crmid($crmID) type($type)";
}


/**
* setObject() function.
* deletes a SugarCRM object
*
* @param String $crmID    CRM-id of the object
* @param String $type     Type of the object
* @return boolean|String
*/
function deleteObject($crmID, $type) {
    global $current_user, $timedate;

    // array of sugar-objects
    if     ($type == "tasks") $seed = new Task();
    elseif ($type == "contacts") $seed = new Contact();
    elseif ($type == "notes")      $seed = new Note();
    elseif ($type == "calls")      $seed = new Call();
    elseif ($type == "meetings") $seed = new Meeting();
    else return "error: object type '$type' not implemented";

    if (isset($seed)) {
        // load object
        if ($crmID)
            $objf = $seed->retrieve($crmID);

        // check, if obj loaded correctly
        if (isset($objf)) {
            $seed->mark_deleted($crmID);
            return true;
        }
    }
    return "error: object could not instantiated : crmid($crmID) type($type)";;
}


?>