<?php
/**
 * Zarafa Z-Merge SugarCRM Connector
 *
 * Copyright (C) 2005 - 2008  Zarafa B.V.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License, version 3,
 * and under the terms of the GNU General Public License, version 3,
 * as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU (Affero) General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * and the GNU General Public License along with this program.
 * If not, see <http://www.gnu.org/licenses/>.
 *
 * Created on 22.04.2008 by Sebastian Kummer & Manfred Kutas
 *
 * @package Z-Merge
 * @subpackage SugarCRM-Connector
 */


/**
 * Build nuSoap objects
 */

$NAMESPACE = 'http://www.zarafa.com/z-merge/sugar';
$server = new soap_server();

$server->configureWSDL('sugarzmergeconnectorsoap', $NAMESPACE, $sugar_config['site_url'].'/z-merge/connector.php');


/**
 * Define XML types
 */
$server->wsdl->addComplexType(
    'name_value',
    'complexType',
        'struct',
        'all',
        '',
        array(
            'name'=>array('name'=>'name', 'type'=>'xsd:string'),
            'value'=>array('name'=>'value', 'type'=>'xsd:string'),
        )
);
$server->wsdl->addComplexType(
    'name_value_list',
    'complexType',
        'array',
        '',
        'SOAP-ENC:Array',
    array(),
    array(
        array('ref'=>'SOAP-ENC:arrayType', 'wsdl:arrayType'=>'tns:name_value[]')
    ),
    'tns:name_value'
);

// lowest level of objects
$server->wsdl->addComplexType(
    'sugar_object',
    'complexType',
    'struct',
    'all',
    '',
    array(
        'objtype' => array('name'=>'objtype', 'type'=>'xsd:string'),
        'name_value_array' => array('name'=>'name_value_array', 'type'=>'tns:name_value_list')
    )
);

// array of SugarCRM-Objects
$server->wsdl->addComplexType(
    'sugar_object_array',
    'complexType',
    'array',
    '',
    'SOAP-ENC:Array',
    array(),
    array(
        array('ref'=>'SOAP-ENC:arrayType','wsdl:arrayType'=>'tns:sugar_object[]')
    ),
    'tns:sugar_object'
);


// change object HAS an array of sugar-Objects (if one modification modifies various SugarCRM-Objects)
$server->wsdl->addComplexType(
    'change_object',
    'complexType',
    'struct',
    'all',
    '',
    array(
        'modid' => array('name'=>'modid', 'type'=>'xsd:string'),
        'id' => array('name'=>'id','type'=>'xsd:string'),
        'userid' => array('name'=>'userid','type'=>'xsd:string'),
        'uid' => array('name'=>'uid','type'=>'xsd:string'),
        'modtime' => array('name'=>'modtime','type'=>'xsd:long'),
        'objtype' => array('name'=>'objtype','type'=>'xsd:string'),
        'delete' => array('name'=>'delete','type'=>'xsd:boolean'),
        'data' => array('name'=>'data','type'=>'tns:sugar_object_array'),
        'morechanges' => array('name'=>'objtype','type'=>'xsd:boolean'),
    )
);

// change-object-array -- returned to the Z-Merge main programm
$server->wsdl->addComplexType(
    'change_object_array',
    'complexType',
    'array',
    '',
    'SOAP-ENC:Array',
    array(),
    array(
        array('ref'=>'SOAP-ENC:arrayType','wsdl:arrayType'=>'tns:change_object[]')
    ),
    'tns:change_object'
);


$server->wsdl->addComplexType(
    'return_object',
    'complexType',
    'struct',
    'all',
    '',
    array(
        'modid' => array('name'=>'modid', 'type'=>'xsd:string'),
        'id' => array('name'=>'id','type'=>'xsd:string'),
        'userid' => array('name'=>'userid','type'=>'xsd:string'),
        'uid' => array('name'=>'uid','type'=>'xsd:string'),
        'status' => array('name'=>'status', 'type'=>'xsd:string'),
        'message' => array('name'=>'message', 'type'=>'xsd:string'),
    )
);


$server->wsdl->addComplexType(
    'return_object_array',
    'complexType',
    'array',
    '',
    'SOAP-ENC:Array',
    array(),
    array(
        array('ref'=>'SOAP-ENC:arrayType','wsdl:arrayType'=>'tns:return_object[]')
    ),
    'tns:return_object'
);



/**
 * Register SOAP functions
 */
$server->register(    'getChanges',
                    array('session'=>'xsd:string', 'modid'=>'xsd:string', 'completeChanges'=>'xsd:boolean'),
                    array('result'=>'tns:change_object_array'),
                    $NAMESPACE
);

$server->register(    'setChanges',
                    array('session'=>'xsd:string', 'modid'=>'xsd:string', 'isConflict'=>'xsd:boolean'),
                    array('newobj'=>'tns:return_object_array'),
                    $NAMESPACE
);

$server->register(    'ping',
                    array (),                                                   // input parameters
                    array ('alive' => 'xsd:boolean'),                           // output parameters
                    $NAMESPACE,                                                 // namespace
                    $NAMESPACE.'#ping',                                         // soapaction
                    'rpc',                                                      // style
                    'encoded',                                                  // use
                    'Returns wheather the web and db server are responding'     // documentation
);


?>